/**
  *  =====================================================================
  *  TrackShape.java: Construct track shape element ...
  *
  *     @param x the left of the bounding rectangle
  *     @param y the top of the bounding rectangle
  *     @param width the width of the bounding rectangle
  *
  *  =====================================================================
  */

package shape;

import java.util.*;
import java.awt.*;
import java.awt.geom.*;

public class TrackShape extends CompoundShape {
   GeneralPath star = new GeneralPath();
   boolean DEBUG = false;

   public TrackShape( int x, int y, double dX, double dY, double width ) {
      double dX0, dY0;
      double dX1, dY1;
      double dX2, dY2;
      double dX3, dY3;
      boolean DEBUG = false;

      // Save corner reference point for shape ...

      this.x = (double) x;
      this.y = (double) y;

      // Compute orientation of the track element ...

      double dAngle = getAngle( dX, dY );

      // Compute corner points for the track element ....

      if (DEBUG == true) {
         System.out.printf("*** In TrackShape(): (dX,dY) = ( %5.1f, %5.1f )\n", dX, dY );
         System.out.printf("***                :   angle = %8.3f\n", dAngle );
      }

      // Set the initial coordinate of the GeneralPath

      dX0 =  (width/2.0)*Math.sin(dAngle);
      dY0 = -(width/2.0)*Math.cos(dAngle);

      dX1 = -(width/2.0)*Math.sin(dAngle);
      dY1 =  (width/2.0)*Math.cos(dAngle);

      dX2 = dX - (width/2.0)*Math.sin(dAngle);
      dY2 = dY + (width/2.0)*Math.cos(dAngle);

      dX3 = dX + (width/2.0)*Math.sin(dAngle);
      dY3 = dY - (width/2.0)*Math.cos(dAngle);

      // Create the track polygon ....

      star.moveTo( dX0, dY0 );
      star.lineTo( dX1, dY1 );
      star.lineTo( dX2, dY2 );
      star.lineTo( dX3, dY3 );
      star.closePath();
        
      add(star);
   }

   // Compute angle for coordinates in four quadrants ....

   public double getAngle( double dX, double dY ) {
       double angle = 0.0;

       if ( dY >= 0.0 && dX >= 0.0 )
            angle = Math.atan( dY/dX );
       if ( dY >= 0.0 && dX < 0.0 )
            angle = Math.PI + Math.atan( dY/dX );
       if ( dY  < 0.0 && dX < 0.0 )
            angle = Math.PI + Math.atan( dY/dX );
       if ( dY  < 0.0 && dX >= 0.0 )
            angle = 2*Math.PI + Math.atan( dY/dX );

       return angle;
   }

   // Test to see if a point is contained within the track element.

   public boolean contains( Point2D pt ) {
      double xCoord = pt.getX() - this.x;
      double yCoord = this.y - pt.getY();
      Point2D.Double testPoint = new Point2D.Double( xCoord, yCoord );

      // Set shapeActive to true when cursor is inside shape ...

      if ( star.contains ( testPoint ) == true )
         shapeActive = true;  
      else 
         shapeActive = false;  

      if (DEBUG == true) {
         System.out.println("*** In CompoundShape.contains(): name = " + sName );
         System.out.println("***                            : xCoord = " + xCoord );
         System.out.println("***                            : yCoord = " + yCoord );
         System.out.println("***                            : inside = " + shapeActive );
      }

      return star.contains ( testPoint );
   }
}
